@@ -1,6 +1,7 @@
 RSpec::Support.require_rspec_core "backtrace_formatter"
 RSpec::Support.require_rspec_core "ruby_project"
 RSpec::Support.require_rspec_core "formatters/deprecation_formatter"
+RSpec::Support.require_rspec_core "output_wrapper"
 
 module RSpec
   module Core
@@ -141,7 +142,7 @@ module RSpec
 
       # Determines where deprecation warnings are printed.
       # Defaults to `$stderr`.
-      # @return [IO, String] IO to write to or filename to write to
+      # @return [IO, String] IO or filename to write to
       define_reader :deprecation_stream
 
       # Determines where deprecation warnings are printed.
@@ -199,6 +200,10 @@ module RSpec
       # The exit code to return if there are any failures (default: 1).
       add_setting :failure_exit_code
 
+      # @macro add_setting
+      # Whether or not to fail when there are no RSpec examples (default: false).
+      add_setting :fail_if_no_examples
+
       # @macro define_reader
       # Indicates files configured to be required.
       define_reader :requires
@@ -214,7 +219,7 @@ module RSpec
       define_reader :output_stream
 
       # Set the output stream for reporter.
-      # @attr value [IO] value for output, defaults to $stdout
+      # @attr value [IO, String] IO to write to or filename to write to, defaults to $stdout
       def output_stream=(value)
         if @reporter && !value.equal?(@output_stream)
           warn "RSpec's reporter has already been initialized with " \
@@ -223,6 +228,7 @@ module RSpec
             "it to take effect. (Called from #{CallerFilter.first_non_rspec_line})"
         else
           @output_stream = value
+          output_wrapper.output = @output_stream
         end
       end
 
@@ -394,6 +400,7 @@ module RSpec
       add_setting :max_displayed_failure_line_count
 
       # @private
+      # @deprecated Use {#color_mode} = :on, instead of {#color} with {#tty}
       add_setting :tty
       # @private
       attr_writer :files_to_run
@@ -422,9 +429,11 @@ module RSpec
         @files_or_directories_to_run = []
         @loaded_spec_files = Set.new
         @color = false
+        @color_mode = :automatic
         @pattern = '**{,/*/**}/*_spec.rb'
         @exclude_pattern = ''
         @failure_exit_code = 1
+        @fail_if_no_examples = false
         @spec_files_loaded = false
 
         @backtrace_formatter = BacktraceFormatter.new
@@ -473,8 +482,14 @@ module RSpec
       # @private
       def reset
         @spec_files_loaded = false
+        reset_reporter
+      end
+
+      # @private
+      def reset_reporter
         @reporter = nil
         @formatter_loader = nil
+        @output_wrapper = nil
       end
 
       # @private
@@ -773,24 +788,54 @@ module RSpec
         @backtrace_formatter.full_backtrace = true_or_false
       end
 
-      # Returns the configuration option for color, but should not
-      # be used to check if color is supported.
+      # Enables color output if the output is a TTY.  As of RSpec 3.6, this is
+      # the default behavior and this option is retained only for backwards
+      # compatibility.
       #
+      # @deprecated No longer recommended because of complex behavior. Instead,
+      #   rely on the fact that TTYs will display color by default, or set
+      #   {#color_mode} to :on to display color on a non-TTY output.
+      # @see color_mode
       # @see color_enabled?
       # @return [Boolean]
       def color
         value_for(:color) { @color }
       end
 
+      # The mode for determining whether to display output in color. One of:
+      #
+      # - :automatic - the output will be in color if the output is a TTY (the
+      #   default)
+      # - :on - the output will be in color, whether or not the output is a TTY
+      # - :off - the output will not be in color
+      #
+      # @see color_enabled?
+      # @return [Boolean]
+      def color_mode
+        value_for(:color_mode) { @color_mode }
+      end
+
       # Check if color is enabled for a particular output.
       # @param output [IO] an output stream to use, defaults to the current
       #        `output_stream`
       # @return [Boolean]
       def color_enabled?(output=output_stream)
-        output_to_tty?(output) && color
+        case color_mode
+        when :on then true
+        when :off then false
+        else # automatic
+          output_to_tty?(output) || (color && tty?)
+        end
       end
 
+      # Set the color mode.
+      attr_writer :color_mode
+
       # Toggle output color.
+      #
+      # @deprecated No longer recommended because of complex behavior. Instead,
+      #   rely on the fact that TTYs will display color by default, or set
+      #   {:color_mode} to :on to display color on a non-TTY output.
       attr_writer :color
 
       # @private
@@ -813,19 +858,22 @@ module RSpec
       end
 
       # @overload add_formatter(formatter)
+      # @overload add_formatter(formatter, output)
       #
-      # Adds a formatter to the formatters collection. `formatter` can be a
-      # string representing any of the built-in formatters (see
-      # `built_in_formatter`), or a custom formatter class.
+      # @param formatter [Class, String] formatter to use. Can be any of the
+      #   string values supported from the CLI (`p`/`progress`,
+      #   `d`/`doc`/`documentation`, `h`/`html`, or `j`/`json`) or any
+      #   class that implements the formatter protocol and has registered
+      #   itself with RSpec as a formatter.
+      # @param output [String, IO] where the formatter will write its output.
+      #   Can be an IO object or a string path to a file. If not provided,
+      #   the configured `output_stream` (`$stdout`, by default) will be used.
       #
-      # ### Note
+      # Adds a formatter to the set RSpec will use for this run.
       #
-      # For internal purposes, `add_formatter` also accepts the name of a class
-      # and paths to use for output streams, but you should consider that a
-      # private api that may change at any time without notice.
-      def add_formatter(formatter_to_use, *paths)
-        paths << output_stream if paths.empty?
-        formatter_loader.add formatter_to_use, *paths
+      # @see RSpec::Core::Formatters::Protocol
+      def add_formatter(formatter, output=output_wrapper)
+        formatter_loader.add(formatter, output)
       end
       alias_method :formatter=, :add_formatter
 
@@ -890,7 +938,7 @@ module RSpec
         @reporter_buffer || @reporter ||=
           begin
             @reporter_buffer = DeprecationReporterBuffer.new
-            formatter_loader.setup_default output_stream, deprecation_stream
+            formatter_loader.prepare_default output_wrapper, deprecation_stream
             @reporter_buffer.play_onto(formatter_loader.reporter)
             @reporter_buffer = nil
             formatter_loader.reporter
@@ -934,7 +982,9 @@ module RSpec
           if (path = example_status_persistence_file_path)
             begin
               ExampleStatusPersister.load_from(path).inject(statuses) do |hash, example|
-                hash[example.fetch(:example_id)] = example.fetch(:status)
+                status = example[:status]
+                status = UNKNOWN_STATUS unless VALID_STATUSES.include?(status)
+                hash[example.fetch(:example_id)] = status
                 hash
               end
             rescue SystemCallError => e
@@ -954,6 +1004,15 @@ module RSpec
       # @private
       FAILED_STATUS = "failed".freeze
 
+      # @private
+      PASSED_STATUS = "passed".freeze
+
+      # @private
+      PENDING_STATUS = "pending".freeze
+
+      # @private
+      VALID_STATUSES = [UNKNOWN_STATUS, FAILED_STATUS, PASSED_STATUS, PENDING_STATUS]
+
       # @private
       def spec_files_with_failures
         @spec_files_with_failures ||= last_run_statuses.inject(Set.new) do |files, (id, status)|
@@ -1432,7 +1491,7 @@ module RSpec
 
         files_to_run.uniq.each do |f|
           file = File.expand_path(f)
-          load file
+          load_spec_file_handling_errors(file)
           loaded_spec_files << file
         end
 
@@ -1658,7 +1717,7 @@ module RSpec
       #       mocks.patch_marshal_to_support_partial_doubles = false
       #     end
       #
-      #     config.mock_with :rspec do |expectations|
+      #     config.expect_with :rspec do |expectations|
       #       expectations.syntax = :expect
       #     end
       #   end
@@ -1721,7 +1780,7 @@ module RSpec
           return unless example_or_group_meta.key?(:example_group)
 
           # Ensure the callback only fires once.
-          @derived_metadata_blocks.items_for(specified_meta).delete(callback)
+          @derived_metadata_blocks.delete(callback, specified_meta)
 
           block.call
         end
@@ -1859,6 +1918,14 @@ module RSpec
 
     private
 
+      def load_spec_file_handling_errors(file)
+        load file
+      rescue Support::AllExceptionsExceptOnesWeMustNotRescue => ex
+        relative_file = Metadata.relative_path(file)
+        reporter.notify_non_example_exception(ex, "An error occurred while loading #{relative_file}.")
+        RSpec.world.wants_to_quit = true
+      end
+
       def handle_suite_hook(scope, meta)
         return nil unless scope == :suite
 
@@ -1882,6 +1949,12 @@ module RSpec
             hook.run(context)
           rescue Support::AllExceptionsExceptOnesWeMustNotRescue => ex
             context.set_exception(ex)
+
+            # Do not run subsequent `before` hooks if one fails.
+            # But for `after` hooks, we run them all so that all
+            # cleanup bits get a chance to complete, minimizing the
+            # chance that resources get left behind.
+            break if hooks.equal?(@before_suite_hooks)
           end
         end
       end
@@ -1985,8 +2058,12 @@ module RSpec
         )
       end
 
+      def output_wrapper
+        @output_wrapper ||= OutputWrapper.new(output_stream)
+      end
+
       def output_to_tty?(output=output_stream)
-        tty? || (output.respond_to?(:tty?) && output.tty?)
+        output.respond_to?(:tty?) && output.tty?
       end
 
       def conditionally_disable_mocks_monkey_patching
