@@ -1,3 +1,4 @@
+RSpec::Support.require_rspec_core "formatters/console_codes"
 RSpec::Support.require_rspec_core "formatters/exception_presenter"
 RSpec::Support.require_rspec_core "formatters/helpers"
 RSpec::Support.require_rspec_core "shell_escape"
@@ -51,8 +52,7 @@ module RSpec::Core
                   FailedExampleNotification
                 end
 
-        exception_presenter = Formatters::ExceptionPresenter::Factory.new(example).build
-        klass.new(example, exception_presenter)
+        klass.new(example)
       end
 
       private_class_method :new
@@ -200,9 +200,15 @@ module RSpec::Core
         @exception_presenter.fully_formatted(failure_number, colorizer)
       end
 
+      # @return [Array<string>] The failure information fully formatted in the way that
+      #   RSpec's built-in formatters emit, split by line.
+      def fully_formatted_lines(failure_number, colorizer=::RSpec::Core::Formatters::ConsoleCodes)
+        @exception_presenter.fully_formatted_lines(failure_number, colorizer)
+      end
+
     private
 
-      def initialize(example, exception_presenter=Formatters::ExceptionPresenter.new(example.execution_result.exception, example))
+      def initialize(example, exception_presenter=Formatters::ExceptionPresenter::Factory.new(example).build)
         @exception_presenter = exception_presenter
         super(example)
       end
@@ -281,8 +287,12 @@ module RSpec::Core
     # @attr pending_examples [Array<RSpec::Core::Example>] the pending examples
     # @attr load_time [Float] the number of seconds taken to boot RSpec
     #                         and load the spec files
+    # @attr errors_outside_of_examples_count [Integer] the number of errors that
+    #                                                  have occurred processing
+    #                                                  the spec suite
     SummaryNotification = Struct.new(:duration, :examples, :failed_examples,
-                                     :pending_examples, :load_time)
+                                     :pending_examples, :load_time,
+                                     :errors_outside_of_examples_count)
     class SummaryNotification
       # @api
       # @return [Fixnum] the number of examples run
@@ -308,6 +318,11 @@ module RSpec::Core
         summary = Formatters::Helpers.pluralize(example_count, "example")
         summary << ", " << Formatters::Helpers.pluralize(failure_count, "failure")
         summary << ", #{pending_count} pending" if pending_count > 0
+        if errors_outside_of_examples_count > 0
+          summary << ", "
+          summary << Formatters::Helpers.pluralize(errors_outside_of_examples_count, "error")
+          summary << " occurred outside of examples"
+        end
         summary
       end
 
@@ -321,7 +336,7 @@ module RSpec::Core
       #                          specific colors.
       # @return [String] A colorized results line.
       def colorized_totals_line(colorizer=::RSpec::Core::Formatters::ConsoleCodes)
-        if failure_count > 0
+        if failure_count > 0 || errors_outside_of_examples_count > 0
           colorizer.wrap(totals_line, RSpec.configuration.failure_color)
         elsif pending_count > 0
           colorizer.wrap(totals_line, RSpec.configuration.pending_color)
